
// アプリケーション情報
var appInfo = {
	// 最後に示していたマップの中央位置
	lastLocation : {
		latitude:  0, // 緯度
		longitude: 0  // 経度
	},

	currentAddressTaskId : 0, // 現在の住所取得タスクの ID
};

// 背景色を変更
Titanium.UI.setBackgroundColor( '#000000' );

////////////////////////////////////////////////////////////////////////////////
//								コントロールの生成
////////////////////////////////////////////////////////////////////////////////

/** メイン ウィンドウ。 */
var mainWindow = Titanium.UI.createWindow({  
    backgroundColor: '#666666'
});

/** Go! ボタン。 */
var buttonGo = Titanium.UI.createButton( {
	top:     2,
	right:   2,
	width:  40,
	height: 40,
	title: 'Go!'
} );

/** 住所入力欄となるテキスト フィールドです。 */
var editAddress = Titanium.UI.createTextField( {
	top:            2,
	left:           2,
	right:         44,
	height:        40,
	color:         '#336699',
	hintText:      "住所を入力します",
	keyboardType:  Titanium.UI.KEYBOARD_DEFAULT,
	returnKeyType: Titanium.UI.RETURNKEY_DEFAULT,
	borderStyle:   Titanium.UI.INPUT_BORDERSTYLE_ROUNDED
} );

/** マップを操作するためのビューです。 */
var mapView = Titanium.Map.createView( {
	top:          44,
	bottom:       42,
	left:          2,
	right:         2,
	mapType:      Titanium.Map.STANDARD_TYPE,
	region :      { latitude:35.6894876, longitude: 139.6917064 },
	animate:      true,
	regionFit:    true,
	userLocation: true
} );

/** マップ中央の住所を表示するためのラベルです。 */
var labelAddress = Titanium.UI.createLabel( {
	bottom:       2,
	left:         2,
	right:        2,
	height:       40,
	text:         '住所',
	textAlign:    'center',
	color:        '#FFF',
	shadowColor:  '#444',
	shadowOffset: { x:0, y:1 }
} );

////////////////////////////////////////////////////////////////////////////////
//									イベント
////////////////////////////////////////////////////////////////////////////////

/**
 * マップを指定された住所の位置へ移動します。
 * 
 * @param {Object} address 住所。
 */
function moveToAddress( address ) {
	var url = 'http://maps.google.com/maps/api/geocode/json?address=' + address + '&sensor=true';
	var xhr = Titanium.Network.createHTTPClient();
	
	xhr.open( 'GET', url );
	Ti.API.info( '>>> Geocoder URL: ' + url );
	
	// Geocoder API から返された位置情報を設定
	xhr.onload = function() {
		var json = eval( '(' + xhr.responseText + ')' );
		if( json.results[ 0 ] ) {
			var location = json.results[ 0 ].geometry.location;
			Ti.API.info( '>>> Geocoder results : Latitude = ' + location.lat + ', Longitude = ' + location.lng );

			mapView.location = {
				latitude:       location.lat,
				longitude:      location.lng,
				latitudeDelta:  0.01,
				longitudeDelta: 0.01
			};
		}
	};
	
	xhr.send();
}

/**
 * 領域変更がおこなわれたときに呼び出されます。
 * 
 * @param {Object} latitude  緯度。
 * @param {Object} longitude 経度。
 */
function onRegionChanged( latitude, longitude ) {

	// このイベントはマップの拡縮でも発生するので、必ず移動を判定する	
	if( appInfo.lastLocation.latitude == latitude && appInfo.lastLocation.longitude == longitude ) { return; }
	appInfo.lastLocation.latitude  = latitude;
	appInfo.lastLocation.longitude = longitude;

	// 領域変更はマップの移動中、大量に呼び出される。また、住所取得のコストが大きいので、
	// 住所更新の処理は識別子を割り当てたうえで遅延実行し、最後におこなわれたものだけ確定させる。
	//
	var id = ++appInfo.currentAddressTaskId;
	setTimeout( function() {
		if( appInfo.currentAddressTaskId != id ) { return; }
	
		var url = 'http://maps.google.com/maps/api/geocode/json?latlng=' + latitude + ',' + longitude + '&language=ja&sensor=true';
		var xhr = Titanium.Network.createHTTPClient();
		
		xhr.open( 'GET', url );
		Ti.API.info( '>>> Geocoder URL: ' + url );
		
		// Geocoder API から返された住所を設定
		xhr.onload = function() {
			var json = eval( '(' + xhr.responseText + ')' );
			var address = ( json.results[ 0 ] ? json.results[ 0 ].formatted_address : undefined );
			if( address ) {
				Ti.API.info( '>>> Geocoder results : ' + address );
				labelAddress.text = address;
			}	
		};
		
		xhr.send();	
	},
	200 );
}

// Go! ボタン押下イベント
buttonGo.addEventListener( 'click', function( e ) {
	if( editAddress.value ) {
		moveToAddress( editAddress.value );
	}
} );

// マップの表示領域が更新されたときのイベント
mapView.addEventListener( 'regionChanged', function( e ) {
	Ti.API.info( '>>> on region chnaged. ' );
	onRegionChanged( e.latitude, e.longitude );
} );

////////////////////////////////////////////////////////////////////////////////
//							ウィンドウの初期化
////////////////////////////////////////////////////////////////////////////////

mainWindow.add( buttonGo     );
mainWindow.add( editAddress  );
mainWindow.add( mapView      );
mainWindow.add( labelAddress );
mainWindow.open();
